<?php

namespace Litecms\Representative\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Representative\Http\Requests\RepresentativeRequest;
use Litecms\Representative\Interfaces\RepresentativeRepositoryInterface;
use Litecms\Representative\Models\Representative;

/**
 * Resource controller class for representative.
 */
class RepresentativeResourceController extends BaseController
{

    /**
     * Initialize representative resource controller.
     *
     * @param type RepresentativeRepositoryInterface $representative
     *
     * @return null
     */
    public function __construct(RepresentativeRepositoryInterface $representative)
    {
        parent::__construct();
        $this->repository = $representative;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Representative\Repositories\Criteria\RepresentativeResourceCriteria::class);
    }

    /**
     * Display a list of representative.
     *
     * @return Response
     */
    public function index(RepresentativeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Representative\Repositories\Presenter\RepresentativePresenter::class)
                ->$function();
        }

        $representatives = $this->repository->paginate();

        return $this->response->title(trans('representative::representative.names'))
            ->view('representative::representative.index', true)
            ->data(compact('representatives'))
            ->output();
    }

    /**
     * Display representative.
     *
     * @param Request $request
     * @param Model   $representative
     *
     * @return Response
     */
    public function show(RepresentativeRequest $request, Representative $representative)
    {

        if ($representative->exists) {
            $view = 'representative::representative.show';
        } else {
            $view = 'representative::representative.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('representative::representative.name'))
            ->data(compact('representative'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new representative.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RepresentativeRequest $request)
    {

        $representative = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('representative::representative.name')) 
            ->view('representative::representative.create', true) 
            ->data(compact('representative'))
            ->output();
    }

    /**
     * Create new representative.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RepresentativeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $representative                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('representative::representative.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('representative/representative/' . $representative->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/representative/representative'))
                ->redirect();
        }

    }

    /**
     * Show representative for editing.
     *
     * @param Request $request
     * @param Model   $representative
     *
     * @return Response
     */
    public function edit(RepresentativeRequest $request, Representative $representative)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('representative::representative.name'))
            ->view('representative::representative.edit', true)
            ->data(compact('representative'))
            ->output();
    }

    /**
     * Update the representative.
     *
     * @param Request $request
     * @param Model   $representative
     *
     * @return Response
     */
    public function update(RepresentativeRequest $request, Representative $representative)
    {
        try {
            $attributes = $request->all();

            $representative->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('representative::representative.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('representative/representative/' . $representative->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('representative/representative/' . $representative->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the representative.
     *
     * @param Model   $representative
     *
     * @return Response
     */
    public function destroy(RepresentativeRequest $request, Representative $representative)
    {
        try {

            $representative->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('representative::representative.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('representative/representative/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('representative/representative/' . $representative->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple representative.
     *
     * @param Model   $representative
     *
     * @return Response
     */
    public function delete(RepresentativeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('representative::representative.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('representative/representative'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/representative/representative'))
                ->redirect();
        }

    }

    /**
     * Restore deleted representatives.
     *
     * @param Model   $representative
     *
     * @return Response
     */
    public function restore(RepresentativeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('representative::representative.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/representative/representative'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/representative/representative/'))
                ->redirect();
        }

    }

}
